from parsel import Selector
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import time
import os
import requests
import enum
from slugify import slugify


class ERRORS(enum.IntEnum):
    """
    Contains constant errors for Gplaycli
    """
    SUCCESS = 0
    ERROR_FETCHING_DOWNLOAD = 15
    NO_DOWNLOAD_LINK_FOUND = 6
    KEYRING_NOT_INSTALLED = 10
    CANNOT_LOGIN_GPLAY = 15


class Apkcombo():
    """docstring for ."""

    def __init__(self, args=None):
        self.driver_path = "G:\chromedriver\chromedriver.exe"
        self.download_folder = "G:\scraping"
        if args.dpath is not None:
            self.dpath = args.dpath
        if args.folder is not None:
            self.download_folder = args.folder[0]
        capa = DesiredCapabilities.CHROME
        capa["pageLoadStrategy"] = "none"
        proxy = "203.13.32.211:80"
        # configure chrome browser to not load images and javascript
        options = webdriver.ChromeOptions()
        # set window size to native GUI size
        options.add_argument("--window-size=1920,1080")
        options.add_argument("start-maximized")  # ensure window is full-screen
        options.add_argument('disable-infobars')
        # options.add_argument('--disable-gpu')
        options.add_argument('--headless')
        # options.add_argument('--proxy-server=%s' % proxy)
        options.add_experimental_option(
            # this will disable image loading
            "prefs", {"useAutomationExtension": False,
                      "intl.accept_languages": "en,en_US"}
        )
        options.add_experimental_option(
            "excludeSwitches", ["enable-logging", "enable-automation"])
        s = Service(self.driver_path)
        self.driver = webdriver.Chrome(
            service=s, options=options, desired_capabilities=capa)
        self.version = None
        self.filename = None

    def getDownloadLink(self, appID=None):
        self.driver.get("https://apkcombo.com/downloader/")
        # wait for page to load
        element = WebDriverWait(driver=self.driver, timeout=30).until(
            EC.presence_of_element_located((By.ID, 'package'))
        )
        # print("Search input showing")
        input = self.driver.find_element(By.NAME, "package")
        input.send_keys(appID)
        sumbitBtn = self.driver.find_element(By.ID, 'apksumbit')

        # Sumbit button
        sumbitBtn.click()
        # wait for URL to change with 15 seconds timeout

        linksTab = WebDriverWait(driver=self.driver, timeout=30).until(
            EC.presence_of_element_located((By.ID, 'apkcombo-tab'))
        )
        # print(element);
        elements = self.driver.find_elements(
            By.CSS_SELECTOR, '.file-list li a')
        links = [elem.get_attribute('href') for elem in elements]
        # print("Link has been fetched {link}".format(link = links[0]))
        # Download file
        # time.sleep(15)
        filterApks = []
        for link in links:
            if (".apk" in link and "&ip=" in link):
                filterApks.append(link)

        data = {}
        dlink = None
        try:
            dlink = filterApks[0]
        except IndexError:
            raise ERRORS.NO_DOWNLOAD_LINK_FOUND
        self.version = self.driver.find_element(
            By.CSS_SELECTOR, '.app_header .version').text
        self.version = slugify(self.version, max_length=20,
                               word_boundary=True, separator=".")
        self.filename = '{filename}.{version}{extension}'.format(
            filename=appID, version=self.version, extension='.apk')
        self.filename = slugify(
            self.filename, max_length=200, word_boundary=True, separator=".")
        self.driver.close()
        data['version'] = self.version
        data['filename'] = self.filename
        data['link'] = dlink
        return data

    def download(self, appID):
        data = {"ok": False, "code": 200, "filename": None, "message": None}
        try:
            link = self.getDownloadLink(appID)["link"]
            r = requests.get(link, allow_redirects=True)
            # Check if download folder exists or create it
            if not os.path.isdir(self.download_folder):
                os.makedirs(self.download_folder, exist_ok=True)
            filepath = os.path.join(self.download_folder, self.filename)
            if os.path.isfile(filepath):
                data["message"] = "File {filename} already exists, skipping.".format(
                    filename=self.filename)
            else:
                open(filepath, 'wb').write(r.content)
                data["message"] = "File has been downloaded successfully {filename}".format(
                    filename=self.filename)
            data["ok"] = True
            data["filename"] = self.filename
        except Exception as e:
            data["message"] = "Failed to download file {filename}".format(
                filename=self.filename)
            data["code"] = 500
        return data



#! /usr/bin/env python3

import os
import sys
import json
import enum
import argparse
from pkg_resources import get_distribution, DistributionNotFound
import logging
from scrapis.apkcombo import Apkcombo
from google_play_scraper import app

try:
	__version__ = '%s [Python%s] ' % (get_distribution('scrapis').version, sys.version.split()[0])
except DistributionNotFound:
	__version__ = 'unknown: scrapis not installed'

logging.basicConfig(level = logging.INFO)
logger  = logging.getLogger(__name__)  # default level is WARNING
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter("[%(levelname)s] %(message)s"))
logger.addHandler(handler)
logger.propagate = False


class ERRORS(enum.IntEnum):
	"""
	Contains constant errors for Gplaycli
	"""
	SUCCESS = 0
	ERROR_FETCHING_DOWNLOAD = 15
	TOKEN_DISPENSER_SERVER_ERROR = 6
	KEYRING_NOT_INSTALLED = 10
	CANNOT_LOGIN_GPLAY = 15

class Scrapper():
	def __init__(self, args=None):
		self.args = args
		self.locale = "en"
		self.country = "us"
		self.folder = None
		self.website = "apkcombo"
		self.driver = None
		if args.locale is not None:
			self.locale = args.locale

		if args.country is not None:
			self.country = args.country

		if args.progress is not None:
			self.progress_bar = args.progress

		if args.website is not None:
			self.website = args.website

		if args.folder is not None:
			self.folder = args.folder[0]

		if args.log is not None:
			self.logging_enable = args.log
		if self.website == "apkcombo":
			self.driver = Apkcombo(args = self.args)
	def dlink(self, appID):
		"""
		Get download link of an app
		"""
		link = {}
		try:
			link = self.driver.getDownloadLink(appID)
		except:
			link = None
		if not link:
			logger.info("No link found")
			return
		print(json.dumps(link))
	def details(self, appID):
		"""
		Get download link of an app
		"""
		details = {}
		try:
			details = app(
			    appID,
			    lang=self.locale, # defaults to 'en'
			    country=self.country, # defaults to 'us'
			)
		except:
			details = {}
		link = {}
		try:
			link = self.driver.getDownloadLink(appID)
		except:
			logger.info("No link found")
		details["download"] = link
		print(json.dumps(details))
def main():
	"""
	Main function.
	Parse command line arguments
	"""
	parser = argparse.ArgumentParser(description="A Google Play Store Apk downloader and manager for command line")
	parser.add_argument('-v',  '--version',				help="Print version number and exit", action='store_true')
	parser.add_argument('-s',  '--search',				help="Search the given string in Google Play Store", metavar="SEARCH")
	parser.add_argument('-d',  '--download',			help="Download the Apps that map given AppIDs", metavar="AppID", nargs="+")
	parser.add_argument('-av', '--append-version',		help="Append versionstring to APKs when downloading", action='store_true')
	parser.add_argument('-a',  '--additional-files',	help="Enable the download of additional files", action='store_true', default=False)
	parser.add_argument('-f',  '--folder',				help="Where to put the downloaded Apks, only for -d command", metavar="FOLDER", nargs=1, default=['.'])
	parser.add_argument('-p',  '--progress',			help="Prompt a progress bar while downloading packages", action='store_true')
	parser.add_argument('-dl',  '--dlink',				help="Get the download link ID", metavar="DLINK")
	parser.add_argument('-dt',  '--details',			help="Get app details", metavar="DETAILS")
	parser.add_argument('-locale',  '--locale',			help="Language to use gplay with", metavar="LOCALE")
	parser.add_argument('-c',  '--country',				help="Country to use gplay with", metavar="COUNTRY")
	parser.add_argument('-w',  '--website',				help="Website to scrapp from", metavar="WEBSITE")
	parser.add_argument('-dpath',  '--dpath',			help="Folder driver path", metavar="WEBSITE")
	parser.add_argument('-r',  '--retry',				help="Number of retries", metavar="RETRIES", default= 0)
	parser.add_argument('-t',  '--timeout',				help="Timout in seconds", metavar="TIMEOUT", default= 0)
	parser.add_argument('-L',  '--log',					help="Enable logging of apps status in separate logging files", action='store_true', default=False)

	if len(sys.argv) < 2:
		sys.argv.append("-h")

	args = parser.parse_args()

	if args.version:
		print(__version__)
		return

	scrapper = Scrapper(args)

	if args.dlink:
		scrapper.dlink(args.dlink)

	if args.details:
		scrapper.details(args.details)

	if args.search:
		scrapper.search(args.search)

	if args.download is not None:
		print(json.dumps(scrapper.driver.download(args.download)))


if __name__ == '__main__':
	main()



from setuptools import setup
import os
import sys

with open("README.md", "r") as fh:
	long_description = fh.read()

setup(name='scrapis',
		version='1.0.3',
		description='Google play downloader command line interface',
		long_description=long_description,
		long_description_content_type="text/markdown",
		author="Redzoua",
		author_email="mthaqafiah@gmail.com",
		url="https://github.com/redzoua/scrapis",
		license="AGPLv3",
		entry_points={
			'console_scripts': [
				'scrapis = scrapis.scrapis:main',
			],
		},
		packages=[
			'scrapis',
		],
		package_dir={
			'scrapis': 'scrapis',
		},
		setup_requires=[
			"wheel"
		],
		install_requires=[
            "setuptools>=61.0",
            "parsel>=1.6.0",
            "selenium>=4.4.0",
            "python-slugify>=6.1.2",
            "requests>=2.28.1",
			"google-play-scraper>=1.2.2",
		],
)
